@@ -1,6 +1,6 @@
 package Package::DeprecationManager;
 BEGIN {
-  $Package::DeprecationManager::VERSION = '0.03';
+  $Package::DeprecationManager::VERSION = '0.05';
 }
 
 use strict;
@@ -74,13 +74,12 @@ sub _build_warn {
 
         my ( $package, undef, undef, $sub ) = caller(1);
 
-        my $skipped = 0;
+        # We want to start two levels back, since we already looked
+        # one level back and found an internal package.
+        my $skipped = 1;
         if ( keys %ignore ) {
             while ( defined $package && $ignore{$package} ) {
-                # We want to start two levels back, since we already looked
-                # one level back and found an internal package.
-                $package = caller($skipped++ + 2);
-                $skipped++;
+                $package = caller($skipped++);
             }
         }
 
@@ -99,8 +98,6 @@ sub _build_warn {
                 && defined $deprecated_at
                 && $compat_version lt $deprecated_at;
 
-        return if $warned{$package}{ $args{feature} };
-
         my $msg;
         if ( defined $args{message} ) {
             $msg = $args{message};
@@ -111,9 +108,13 @@ sub _build_warn {
                 if defined $deprecated_at;
         }
 
-        $warned{$package}{ $args{feature} } = 1;
+        return if $warned{$package}{ $args{feature} }{$msg};
 
-        local $Carp::CarpLevel = $Carp::CarpLevel + 1 + $skipped;
+        $warned{$package}{ $args{feature} }{$msg} = 1;
+
+        # We skip at least two levels. One for this anon sub, and one for the
+        # sub calling it.
+        local $Carp::CarpLevel = $Carp::CarpLevel + $skipped;
 
         Carp::cluck($msg);
     };
@@ -133,7 +134,7 @@ Package::DeprecationManager - Manage deprecation warnings for your distribution
 
 =head1 VERSION
 
-version 0.03
+version 0.05
 
 =head1 SYNOPSIS
 
@@ -196,7 +197,7 @@ in your distribution that can appear on the call stack when a deprecated
 feature is used.
 
 As part of the import process, C<Package::DeprecationManager> will export two
-subroutines into its caller. It proves an C<import()> sub for the caller and a
+subroutines into its caller. It provides an C<import()> sub for the caller and a
 C<deprecated()> sub.
 
 The C<import()> sub allows callers of I<your> class to specify an C<-api_version>
@@ -217,8 +218,10 @@ to the feature name passed in the C<-deprecations> hash.
 If you don't explicitly specify a feature, the C<deprecated()> sub uses
 C<caller()> to identify its caller, using its fully qualified subroutine name.
 
-Deprecation warnings are only issued once for a given package, regardless of
-how many times the deprecated sub/method is called.
+A given deprecation warning is only issued once for a given package. This
+module tracks this based on both the feature name I<and> the error message
+itself. This means that if you provide severaldifferent error messages for the
+same feature, all of those errors will appear.
 
 =head1 BUGS
 
@@ -252,7 +255,7 @@ created as L<Class::MOP::Deprecated> by Goro Fuji.
 
 =head1 AUTHOR
 
-  Dave Rolsky <autarch@urth.org>
+Dave Rolsky <autarch@urth.org>
 
 =head1 COPYRIGHT AND LICENSE
 
